function [V, S] = h2_Gal_GenLyap( A, N, B, m, conv_tol_h2, max_iter_h2, update_n_step )
%H2_GAL_GENLYAP
%  Using a naive block-Arnoldi to compute a subspace aimed at approximating
%  the solution to the generalized Lyapunov equation
%             AX + XA^T + \sum_{i=1}^m N{i}XN{i}^T + BB^T = 0.
%  
%  The space created is
%      V = {B, w_1, w_2, ... , w_k},
%  where w_i satisfies the fixed-point iteration
%      (A + I(w_i^TAw_i) + \sum_{i=1}^m N{i}(w_i^TN{i}w_i))w_i = -R_{i-1}w_i
%  where R_{i-1} is the the residual in the previous space.
%  Returns a space of dimension size(B,2)+(m-1), that is, it does m-1 steps
%
%  [V, S, max_res] = rat_Krylov_type_GenLyap( A, N, B, m, smin, smax, update_n_step )
%
%  Input:
%  A - Matrix defining the Lyapunov operator
%  N - Cell array containing the "generalized" terms of the equation
%  B - Low-rank factor of the "right-hand-side"
%  m - The method does m-1 steps
%  conv_tol_h2 - Tolerence for when the fixed point iteration is considered
%    having converged. Can typcially be set quite high, like 5*10^(-2).
%  max_iter_h2 - Maximum number of iterations of fixed point iteration, in
%    each step of the outer method
%  update_n_step - Allows to update the residual direction r only every
%    update_n_step of the outer iteration. Can be set to 1 if it ise
%    desired to update the residual direction at every iteration
%
%  Output:
%  V - A matrix containing an orthogonal basis of the suggested search
%    space for approximating the generalized Lyapunov equation.
%  s - The values w_i^T A w_i
%


mm = m-1;
n = size(A,1);
R = [];
S = NaN((mm), 1);

V = qr_orth(B); %Orthogonalize and normalize within starting block
In = speye(n);

k = 1;
Ik = speye(k);

    
for i = 1:mm
    fprintf('Iteration: %2.i of %i\n',i, mm)

    [~, R] = update_residual_direction_GenLyap(A, N, B, V, i, update_n_step, [], R, 0);

    % Compute h2 rank-1 update
    Vm = V(:,end);
    Am = Vm'*A*Vm;
    Nm = cell(length(N),1);
    for j = 1:length(N)
        Nm{j} = Vm'*N{j}*Vm;
    end
    s = eig(Am);
    tol = inf;
    iter = 0;
    while ( tol > conv_tol_h2 && iter < max_iter_h2 )
        iter = iter + 1;
        L = - kron(Ik,A) - kron(Am,In);
        for j = 1:length(N)
            L = L - kron(Nm{j},N{j});
        end
        X = reshape(L\reshape(R*Vm,n*k,1), n, k);

        Vm = orth(full(X));
        Am = Vm'*A*Vm;
        for j = 1:length(N)
            Nm{j} = Vm'*N{j}*Vm;
        end
        sold = s;
        s = eig(Am);
        tol = norm(s-sold)/norm(s);
    end
    S(i) = Vm'*A*Vm;
    fprintf('  H2-iteration: Did %i iterations to %d convergence.\n', iter, tol)

    V = qr_orth([V,Vm]);
    
end


