function [ Y ] = inner_solver_GenLyap_oblique( A, N, B, V, W )
%INNER_SOLVER_GENLYAP_OBLIQUE  -  Module for solving the projected problem.
%  Aims at computing the Galerkin projection of the generalized Lyapunov
%  equation,
%  AX + XA^T + \sum_{i=1}^m N{i}XN{i}^T + BB^T = 0,
%  using and oblique projection, W^TAV. Observe that W and V areassumed to
%  be Orthogonal as well as W^TV = I and V^TW = I. Typically for BIRKA one
%  would use W^T=(W^TV)^{-1}W^T.
%  
%  The approxiamtion is X = VYV^T and the Petrov-Galerkin condition enforced
%  W^T(AVYV^T + VYV^TA^T + \sum_{i=1}^m N{i}VYV^TN{i}^T + BB^T)W = 0
%
%  OBS: Allowed to be optimized depending on problem structure!
%
%  [ Y ] = inner_solver_GenLyap_oblique( A, N, B, V, W )
%
%  Input:
%  A - Matrix defining the Lyapunov operator
%  N - Cell array containing the "generalized" terms of the equation
%  B - Low-rank factor of the "right-hand-side"
%  V - Orthogonal matrix containing a basis for the (trial) subpsace in which
%    we seek the solution
%  W - Orthogonal matrix containing a basis for the (test) subspace that we
%    use to enforce the Petrov-Galerkin condition
%
%  Output:
%  Y - The soltion to the projected problem (OBS: the small-scale problem!)
%

  if ~iscell(N); error('N needs to be a cell-array'); end

  
  % Obs: Does not optimize for the symmetric case
  Ap = W'*A*V;
  Np = cell(length(N),1);
  for i = 1:length(N)
    Np{i} = W'*N{i}*V;
  end
  Bp = W'*B;
  
  % Obs: Solve with naive method to allow for most general application
  n = size(Ap,2);
  I = speye(n,n);
  L = kron(I,Ap) + kron(Ap,I);
  for i = 1:length(N)
    L = L + kron(Np{i},Np{i});
  end 
  rhs = (-Bp)*Bp';
  Y = reshape(L\rhs(:), n, n);

end

