function [V, S] = rat_Krylov_type_GenLyap_givenshifts( A, N, B, m, S, update_n_step )
%RAT_KRYLOV_TYPE_GENLYAP_GIVENSHIFTS
%  Using a naive block-Arnoldi to compute a rational Krylov type subspace
%  aimed at approximating the solution to the generalized Lyapunov equation
%             AX + XA^T + \sum_{i=1}^m N{i}XN{i}^T + BB^T = 0.
%  
%  The space created is
%      V = {B, (A-S(1))^{-1}r_1, (A-S(2))^{-1}r_2, ... , (A-S(k))^{-1}r_k},
%  where r_i is the dominating left signular vector of the residual in the
%  previous space. Shifts S are given as input and blindly used in a
%  circular fashion.
%  Returns a space of dimension size(B,2)+(m-1), that is, it does m-1 steps
%
%  [V, S, max_res] = rat_Krylov_type_GenLyap_givenshifts( A, N, B, m, S, update_n_step )
%
%  Input:
%  A - Matrix defining the Lyapunov operator
%  N - Cell array containing the "generalized" terms of the equation
%  B - Low-rank factor of the "right-hand-side"
%  m - The method does m-1 steps
%  S - The vector of prescribed shifts
%  update_n_step - Allows to update the residual direction r only every
%    update_n_step of the outer iteration. Can be set to 1 if it ise
%    desired to update the residual direction at every iteration
%
%  Output:
%  V - A matrix containing an orthogonal basis of the suggested search
%    space for approximating the generalized Lyapunov equation.
%  s - The used shifts
%

if ~iscell(N); error('N needs to be a cell-array'); end;

n = size(A,1);
mm = m-1;
r = [];
R = [];

if issparse(A)
  I = speye(n,n);
else
  I = eye(n,n);
end

nrof_shifts = length(S);

V = qr_orth(B); %Orthogonalize and normalize within starting block

k = 0;
while k < (mm)
  k = k + 1;
  fprintf('Iteration: %2.i of %i\n',k,(mm))

  [r, R] = update_residual_direction_GenLyap(A, N, B, V, k, update_n_step, r, R);
  
  % Use the prescribed shift in a cyclic fashion
  kk = mod(k,nrof_shifts) + 1;
  s = S(kk);
 
%   % Expand the basis
%   Vi = (A - s*I)\r;
%   V = qr_orth([V,Vi]);
  % Update the basis
  if imag(s) == 0 % One real shift
    S(k) = s;
    % Expand the basis
    Vi = (A - s*I)\r;
    V = qr_orth([V,Vi]);
  elseif imag(s) ~= 0 % One complex conjugate shift
    S(k) = s;
    S(k+1) = conj(s);
    % Expand the basis
    Vi = (A - s*I)\r;
    Vi = [real(Vi), imag(Vi)];
    V = qr_orth([V,Vi]);
    k = k+1;
  else
    error('This should not be possible.')
  end
end

end